/// ch341-uart converter now attached to ttyUSB0
extern crate serial;

use std::env;
use std::io;
use std::time::Duration;

use std::io::prelude::*;
use serial::prelude::*;

use ansi_term::Colour::Red as Reads;
use ansi_term::Colour::Blue as Writes;

fn main() {
    let mut port = serial::open("/dev/ttyUSB0").unwrap();
    interact(&mut port).expect("unable to interact with ttyUSB0");
}

fn interact<T: SerialPort>(port: &mut T) -> io::Result<()> {
    (port.reconfigure(&|settings| {
        (settings.set_baud_rate(serial::Baud38400))?;
        //(settings.set_baud_rate(serial::Baud9600))?;
        settings.set_char_size(serial::Bits8);
        settings.set_parity(serial::ParityNone);
        //settings.set_stop_bits(serial::Stop1);
        settings.set_flow_control(serial::FlowNone);
        Ok(())
    }))?;

    (port.set_timeout(Duration::from_millis(1000)))?;

    //send_pkg(port, b"")?;

    let mut buf: Vec<u8> = (0..255).collect();

    //print_bin(port.read(&mut buf[..])?, &buf);

    //(port.write(&buf[..]))?;
    send_pkg(port, b"ETZ")?;
    //print_bin(port.read(&mut buf[..])?, &buf);
    print_bin(port.read(&mut buf[..])?, &buf);

    send_pkg(port, b"ATE0")?;
    print_bin(port.read(&mut buf[..])?, &buf);
    print_bin(port.read(&mut buf[..])?, &buf);
    send_pkg(port, b"ATL0")?;
    print_bin(port.read(&mut buf[..])?, &buf);
    print_bin(port.read(&mut buf[..])?, &buf);
    send_pkg(port, b"ATSP A0")?;
    print_bin(port.read(&mut buf[..])?, &buf);
    print_bin(port.read(&mut buf[..])?, &buf);

    Ok(())
}

fn print_bin(len: usize, buf: &[u8]) -> String {
    let slice = &buf[0..len];
    let mut ascii = String::new();
    ascii.extend(slice.iter().filter(|b| b.is_ascii()).map(|b| char::from(*b)));
    let hex = slice.iter().map(|c| format!("{:x}", c)).collect::<Vec<_>>();
    format!("{} :::::: {:?}", ascii, hex)
}

fn send_pkg<T: SerialPort>(port: &mut T, slice: &[u8]) -> io::Result<usize> {
    println!("{}", Writes.paint(format!("(>>) {}", print_bin(slice.len(), slice) )));
    flush()?;

    let n = port.write(slice)?;
    //let n = n + port.write("\r".as_bytes())?;
    let n = n + port.write("\n\r".as_bytes())?;
    Ok(n)
}

fn flush() -> std::io::Result<()> {
    //use std::io::Write;
    std::io::stdout().flush()
}
